;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x6ab6be5871599445719e5366781ff5b0 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x998a1296bd5be2cd182f52bad646a0a5 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xe0706a4e7ef69f2d0e5b0cf08df21a9a . "./code.cg2js/cg-toolbar.fasl")
(#x70ff635a35ab795afc7f36d6d9442cb6 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x4f4e7cbaf44b501c6d1fc9639d2dd322 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x34912b394b9fabcab27bd8ab7377a377 . "./code.cg2js/cg-class-grid.fasl")
(#x50debe7c14825eb9dc01ce2fc3df5940 . "./code.cg2js/cg-tray.fasl")
(#x2c45fc41237475c745466a859ae996f8 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x083c488fb3b999fea50726a865c8fa8a . "./code.cg2js/cg-class-support.fasl")
(#x8bd1d785252c41a2475ad0b31497799b . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xed1202a2226a706a76af9378866dbfb5 . "./code.cg2js/cg-dropping-outline.fasl")
(#x2e1aa71310f32392b96c3f1652bc0e2c . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x7cee7b5ac85352f2c17725745a60cd71 . "./code.cg2js/cg-choice-list.fasl")
(#x224dc3cca60a98a8e4d40ed9d95148e8 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x79fba36a6647bef5ef864398e76e3d97 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x36b91ae4a6f407e4653837239c2497e6 . "./code.cg2js/cg-timer.fasl")
(#x990b9069227a7e035e8176920c35f947 . "./code.cg2js/cg-directory-dialog.fasl")
(#xe9752623ba6ef05eb4036473f68ee361 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xf448f18716bee3a24fa80d391822eb46 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x9ef791e5a91f0ac2ca80fd6fefd1f211 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x4d7a0717572fbfae4f78a0b8aa7182cb . "./code.cg2js/cg-html-widget.fasl")
(#x50818c106267efd84674d20cce9b2911 . "./code.cg2js/cg-selected-object.fasl")
(#x5de73e836458ebd79be4cede188fce0c . "./code.cg2js/cg-lamp.fasl")
(#x6d4335447238d686566a3a7f68b51864 . "./code.cg2js/cg-scroll-bar.fasl")
(#x4ad4e57cfc6275664c70c8a19aab2c39 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xd30ed0ae08a544cbce1b4e91783401e0 . "./code.cg2js/cg-message-dialog.fasl")
(#xf1cefb15b2456a89a25134856f7afcd3 . "./code.cg2js/cg-dde-utils.fasl")
(#xcf7fa7b39c4280bf456c0ef470cd5c22 . "./code.cg2js/cg-group-box.fasl")
(#xfceed3a1d959aa291cbc8eb358e2084a . "./code.cg2js/cg-find-string-dialog.fasl")
(#x631349bd0e9c44d410b2df6d60b83e43 . "./code.cg2js/cg-split-bar.fasl")
(#x3394eb440056346f4fb766bb0420b173 . "./code.cg2js/cg-radio-button.fasl")
(#xab0ca62861444a8d83c26f21ce8026b6 . "./code.cg2js/cg-base.fasl")
(#x5f6ecf224dbceb1e055f24a9937680b7 . "./code.cg2js/cg-paren-matching.fasl")
(#x0af879326e9cdca9cd0953e1fe909d8d . "./code.cg2js/cg-graphics-context.fasl")
(#xa4ee719952f586b65421486f68e4611d . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xa2f7a6687ffed219ebec95d616a5ea4d . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xb88fb01ddf85508b5bc2ee5aa11c83c0 . "./code.cg2js/cg-plot-widget.fasl")
(#x8413a06a4c603b7708ab30d06fe34013 . "./code.cg2js/cg-dev.fasl")
(#x248984004a0f4953c2acfde8ebd51757 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xdac4f1382730771d319159ac22ee4721 . "./code.cg2js/cg-tab-control.fasl")
(#xbe0665af4663a158e74fc41b040e2a35 . "./code.cg2js/cg-fill-texture.fasl")
(#x9fc8d51df83a9c04c79c3c48ca19a3a1 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x1ab21f50dc140c685d651795f601d6ec . "./code.cg2js/cg-trackbar.fasl")
(#xa7153775071afb91a5cc44dfbd3efa02 . "./code.cg2js/cg-nodes.fasl")
(#x2566b1c4e0a3d6a1708592d442440cb9 . "./code.cg2js/cg-common-control.fasl")
(#x9aea6761425cb6f91c38d18d177232b6 . "./code.cg2js/cg-list-view.fasl")
(#xb2a596bcd8c99081dc0a4e7b456dd057 . "./code.cg2js/cg-rich-edit.fasl")
(#xe17303448ec1213dbb19fbbd710257b5 . "./code.cg2js/cg-project-window.fasl")
(#x7525d73e8010bbe26d0be739f3baa95f . "./code.cg2js/cg-item-list.fasl")
(#x260f1600ce53beda5d90134d862a4699 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x39dfea127b7ffb5258005d662211516a . "./code.cg2js/cg-get-position.fasl")
(#x0398c20eef011ee1a2f63db811a9bb2b . "./code.cg2js/cg-bitmap-pane.fasl")
(#x466633023aac88a4b02cd07450d83a58 . "./code.cg2js/cg-drawable.fasl")
(#x97ffddc86fe3c56c2853ee3947555dd1 . "./code.cg2js/cg-lettered-menu.fasl")
(#x825cbd53ae9a37452511efe60d6d457e . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xc691778d459fb4b664778c7c55a8fd40 . "./code.cg2js/cg-html-dialog.fasl")
(#x35118f1d849d6ada9204b037a4b48197 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x96b34706fe6562313a0ee74dfb8a7b70 . "./code.cg2js/cg-utility-dialog.fasl")
(#x5b2bd59640cf6235caa88c8f9bd6ede1 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xf938a9a0ba73edce596ede9148c3c5e3 . "./code.cg2js/cg-property.fasl")
(#xa5d68aab896425e07b4b630822d33a76 . "./code.cg2js/cg-menu.fasl")
(#xc82bb7555b053938bc52d403cf2a9fff . "./code.cg2js/cg-clipboard.fasl")
(#xc3b6b4cdbd36c741016009d4b2150f1a . "./code.cg2js/cg-toggling-widget.fasl")
(#x0aba866cc7b245348f5b9f78ea0f71fc . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x057008ecc734a31e6ec66abcbbf26e98 . "./code.cg2js/cg-template-string.fasl")
(#x86fed245638100e71eb26cab5a02fcea . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x801c4b2740a07e84aa8f69482f532f21 . "./code.cg2js/cg-os-window.fasl")
(#x60d2c893bb6d522c63da772fcdcaed5a . "./code.cg2js/cg-grid-widget.fasl")
(#x2cf58aee3cbba75d508d668406050b96 . "./code.cg2js/cg-picture-widget.fasl")
(#x9857cf2bcb4a603cba97937a347dce86 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x31c68217f80b39f8255ec9f47782b80a . "./code.cg2js/cg-static-text.fasl")
(#xe24f08d53adb0dce9095ae82e05a1f40 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x0a79bde90144b8cd54635ae04bf60ee6 . "./code.cg2js/cg-text-widget.fasl")
(#x7f3f108fc2829c066b3d191e5a21707b . "./code.cg2js/cg-calendar.fasl")
(#x62da6e75e80c41600412f5d474ce928e . "./code.cg2js/cg-check-box.fasl")
(#x1dd72b4fbfb89f965569bce949cc73dd . "./code.cg2js/cg-acache.fasl")
(#x21b2bef6fe77d00f75ebd9a4d598c0e0 . "./code.cg2js/cg-wrap-string.fasl")
(#xc92c9b4647f63480ee942226d7331611 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xcf5ae185280f2296bb36b4a6bc5cdce9 . "./code.cg2js/cg-font-dialog.fasl")
(#x03d8a4de782dfcbe9a747097dd519d72 . "./code.cg2js/cg-outline.fasl")
(#xc4b37f527164d2bb385bbbccaa87781f . "./code.cg2js/cg-shortcut-menu.fasl")
(#x7467600be142060bc74f051540f2cf20 . "./code.cg2js/cg-status-bar.fasl")
(#x7ab5d337cc04d4f9b6c7d4fd1332e9b7 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x013bebf0c74005ad3e2e8d66f80d42dc . "./code.cg2js/cg-hotspot.fasl")
(#x714487b3cff91a175ffb02f142b3f239 . "./code.cg2js/cg-scaling-stream.fasl")
(#x411042c4c1c143baae0acb8c30e9720b . "./code.cg2js/cg-icon-pixmap.fasl")
(#x89da291378fac1b8006020820b1aacbc . "./code.cg2js/cg-file-dialog.fasl")
(#x985cfab721ec274fa615b4bf4167b90b . "./code.cg2js/cg-color-dialog.fasl")
(#xf554ac42698f5390780511a3afb15e1f . "./code.cg2js/cg-get-pixmap.fasl")
(#xd02a83be701a64b07570ded315de9b78 . "./code.cg2js/cg-object-editor.fasl")
(#xef0626a290d32e9833c7493daf5994a1 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x68af5b007cd315305828c48aba330250 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xe2a4ad45836c15182aca8eb3e72a660e . "./code.cg2js/cg-pixmap.fasl")
(#xdf7c8fe9edfdc83ddc7a96082b9abb96 . "./code.cg2js/cg-intersect.fasl")
(#xdd33299c8764948c1161dcf77112761c . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x6399aa84d68261edf0258f464516ead2 . "./code.cg2js/cg-icon.fasl")
(#xc70da46713a93d7243f61489f9284c2a . "./code.cg2js/cg-yes-no-string.fasl")
(#x469d26b849747a07c0ef59f6c519aa1d . "./code.cg2js/cg-text-or-combo.fasl")
(#xd0279c11dc0a8682e24b17d9e9829ec6 . "./code.cg2js/cg.fasl")
(#x73fc73f893ab35a01152867edd93f356 . "./code.cg2js/cg-dialog-item.fasl")
(#xed088e0a8d61ae727fcd0824ca1a59fd . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x3a9e573b8b9f3a425e4f9aa7b77afe5d . "./code.cg2js/cg-caret.fasl")
(#xff242a4ef56b3133ace7c4a7cd7a945b . "./code.cg2js/cg-editable-text.fasl")
(#xea36c026ef304ce28a78108ab0cda935 . "./code.cg2js/cg-choose-printer.fasl")
(#x2afca1bcd8798506c98319fc59c794ce . "./code.cg2js/cg-yes-no-list.fasl")
(#x91e662ca3a0abe25af18bae3b6ee6c1b . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd80c8348f1f0ef53210c13a2c591336c . "./code.cg2js/cg-tooltip.fasl")
(#xc1abc57f8a233cdb1121fd0f99b04b3c . "./code.cg2js/cg-class-slot-grid.fasl")
(#x68c9d84f71d8b8f16f8823bdf79caad6 . "./code.cg2js/cg-combo-box.fasl")
(#x94d19dfdab9842c4c5af1a79db690283 . "./code.cg2js/cg-curve.fasl")
(#x04c95317c00602ec767138c7c6f8b57f . "./code.cg2js/cg-web-browser.fasl")
(#x641d4412417aa17ab1d49bef8c82edfa . "./code.cg2js/cg-printing.fasl")
(#xd26b4d673c407f9b2d0aa25778a60ec7 . "./code.cg2js/cg-lisp-widget.fasl")
(#xd7f665c9f5693741bb8bea154b62a023 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x972ea5da41bed7b119529513ed83e0ef . "./code.cg2js/cg-palette.fasl")
(#xb69afd8ee939d3f8e823e7e3dbcbacb4 . "./code.cg2js/cg-os-widget.fasl")
(#xa1edd13bd7504832b6b79dcfed4696d9 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x2381e6018c39847381eed716fb62d710 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x855f8d897b5a873d1fc28012f099743f . "./code.cg2js/cg-button.fasl")
(#x7575b1727e1e3da240956622c12115b3 . "./code.cg2js/cg-edit-in-place.fasl")
(#x4bb44493b37324923147a658c7ab5c2b . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xbd966576edab6325be2c4d3eabdfce99 . "./code.cg2js/cg-comtab.fasl")
(#x00a650781d5aac4ac01aeb361fbb9876 . "./code.cg2js/cg-string-dialog.fasl")
(#xbe5496cc61d29fddc6cd5b243b9efd90 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xf350daba8230e0f667758d37f5d4ee6f . "./code.cg2js/cg-up-down-control.fasl")
(#x5d6deff3817dfbac075dea32f34907f9 . "./code.cg2js/cg-layout.fasl")
(#xd1a716f1a720c125165209efccaa7768 . "./code.cg2js/cg-chart-widget.fasl")
(#x7b0ee1e33ab1973499a2f90b0f09674e . "./code.cg2js/cg-lisp-text.fasl")
(#x5b55d294752c57e455253a0afb115411 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xc775dbaaa82d888d27c84a709dd9d446 . "./code.cg2js/cg-progress-indicator.fasl")
(#x57df4a4f46ed2b318eaeb59f14983eee . "./code.idejs/ide-inspector.fasl")
(#xfc196d8727f76e8edb6e2a1d7764a93d . "./code.idejs/ide-grapher.fasl")
(#x6d7018715510d2af966ae241460c1973 . "./code.idejs/ide-base.fasl")
(#x2d8717c868aabdcd571c20bd04796432 . "./code.idejs/ide-keyshort.fasl")
(#x6707be13b8cfdeb23eb8afc2b7da825d . "./code.idejs/ide-dev.fasl")
(#x6cd7f003ecff0e027abe646a987a62db . "./code.idejs/ide-class-browser.fasl")
(#xfc35489af00ce19deec6952917412df3 . "./code.idejs/ide-package-list.fasl")
(#x02950af183150100b9080f5b8d400e04 . "./code.idejs/ide-intro.fasl")
(#x12e7132379add02dc48b461dbaa6250c . "./code.idejs/ide-class-grapher.fasl")
(#x65f182866702c7e10b281b8d99059174 . "./code.idejs/ide-window-list.fasl")
(#xae9a1c9dd096374a3e866da6b1690524 . "./code.idejs/ide-find-definitions.fasl")
(#xc6f7eb78623734630c4fcc68a83b67cb . "./code.idejs/ide-trace.fasl")
(#xb8ec71a87dbcc4e6dc3d07ef107bca73 . "./code.idejs/ide-editor.fasl")
(#x39838b5a79c8458570e2436f4be3591d . "./code.idejs/ide-patch-dialog.fasl")
(#xe6b726b9a3c0ab01bd816e14f716e672 . "./code.idejs/ide-project.fasl")
(#x457ee9a6e08b7a0054fbcdb498c6c95f . "./code.idejs/ide.fasl")
(#x73dacf2f043a395664bbcdc1eb5a751c . "./code.idejs/ide-apropos.fasl")
(#x18a1fbcb10a27bb0bc583ce09f1dbf16 . "./code.idejs/ide-acache.fasl")
(#x486846512db1cfccb95cadf0b2b01f94 . "./code.idejs/ide-clipboard.fasl")
(#xfa079f065bb8369bed14c25d21d36772 . "./code.idejs/ide-process-browser.fasl")
(#x2ff9bcb3bd6f05a3ae4f6738a65bdaf7 . "./code.idejs/ide-find-in-files.fasl")
(#x5406229cc674f0ba6685e36cb459c98c . "./code.idejs/ide-form.fasl")
(#x075940d9bbd24ee9feabc934b26b9d56 . "./code.idejs/ide-navigator.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
